package es.kauron.jstudy.controller;

import es.kauron.jstudy.Main;
import es.kauron.jstudy.model.TestItem;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.SelectionMode;
import javafx.scene.control.TableColumn;
import javafx.scene.control.TableView;
import javafx.stage.Modality;
import javafx.stage.Stage;

import java.io.IOException;
import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

public class TableController implements Initializable {
    @FXML
    private TableView<TestItem> table;
    @FXML
    private TableColumn<TestItem, String> answerCol, questionCol;
    @FXML
    private Button editButton, duplicateButton, swapButton, testSelectionButton, deleteButton;

    private ObservableList<TestItem> data;
    private Controller parent;

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
        table.getSelectionModel().setSelectionMode(SelectionMode.MULTIPLE);
        answerCol.setCellValueFactory(e -> e.getValue().answerProperty());
        questionCol.setCellValueFactory(e -> e.getValue().questionProperty());
        bindButtons();
    }

    private void bindButtons() {
        editButton.disableProperty().bind(table.getSelectionModel().selectedIndexProperty().lessThan(0));
        swapButton.disableProperty().bind(table.getSelectionModel().selectedIndexProperty().lessThan(0));
        deleteButton.disableProperty().bind(table.getSelectionModel().selectedIndexProperty().lessThan(0));
        duplicateButton.disableProperty().bind(table.getSelectionModel().selectedIndexProperty().lessThan(0));
        testSelectionButton.disableProperty().bind(table.getSelectionModel().selectedIndexProperty().lessThan(0));
    }

    void setData(List<TestItem> list, Controller controller) {
        this.data = FXCollections.observableArrayList(list);
        this.parent = controller;
        table.setItems(data);
    }

    List<TestItem> getData() {return data;}

    @FXML
    private void onAddAction(ActionEvent event) {
        try {
            FXMLLoader cargador = new FXMLLoader(Main.class.getResource("view/edit.fxml"));
            Parent pRoot = cargador.load();

            ((EditController) cargador.getController()).setList(data);

            Stage stage = new Stage();
            stage.setTitle("New entry");
            stage.setScene(new Scene(pRoot));
            stage.initModality(Modality.APPLICATION_MODAL);
            stage.showAndWait();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @FXML
    private void onEditAction(ActionEvent event) {
        ObservableList<Integer> list = table.getSelectionModel().getSelectedIndices();
        if (list.size() != 1) return;
        int index = list.get(0);
        try {
            FXMLLoader cargador = new FXMLLoader(Main.class.getResource("view/edit.fxml"));
            Parent root = cargador.load();

            ((EditController) cargador.getController()).setList(table.getItems(), index);

            Stage stage = new Stage();
            stage.setTitle("Editing entry...");
            stage.setScene(new Scene(root));
            stage.initModality(Modality.APPLICATION_MODAL);
            stage.showAndWait();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @FXML
    private void onSwapAction(ActionEvent event) {
        for (TestItem item : table.getSelectionModel().getSelectedItems()) {
            String question = item.getQuestion();
            item.questionProperty().set(item.getAnswer());
            item.answerProperty().set(question);
        }
        table.requestFocus();
    }

    @FXML
    private void onDuplicateAction(ActionEvent event) {
        for (int i : table.getSelectionModel().getSelectedIndices())
            table.getItems().add(new TestItem(table.getItems().get(i)));
        table.requestFocus();
    }

    @FXML
    private void onDeleteAction(ActionEvent event) {
        for (int i : table.getSelectionModel().getSelectedIndices())
            table.getItems().remove(i);
        table.requestFocus();
    }

    @FXML
    private void onTestSelectionAction(ActionEvent event) {
        parent.newTest(table.getSelectionModel().getSelectedItems());
    }

    @FXML
    private void onTestAction(ActionEvent event) {
        parent.newTest(data);
    }
}
