package es.kauron.jstudy.controller;

import com.jfoenix.controls.JFXTextField;
import es.kauron.jstudy.model.AppConfig;
import es.kauron.jstudy.model.TestItem;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.geometry.Insets;
import javafx.scene.control.Label;
import javafx.scene.control.ProgressBar;
import javafx.scene.layout.HBox;

import java.net.URL;
import java.util.List;
import java.util.ResourceBundle;

public class TestController implements Initializable {
    @FXML
    private Label question, prevQuestion, prevAnswer, correctAnswer, progressLabel, correctLabel;
    @FXML
    private JFXTextField answer;
    @FXML
    private ProgressBar progress;
    @FXML
    private HBox feedback;

    private List<TestItem> list;
    private int total, current = -1;
    private int done = 0, errors = 0;

    @Override
    public void initialize(URL url, ResourceBundle resourceBundle) {
    }

    void setList(List<TestItem> list) {
        this.list = list;
        total = list.size();
        progress.setProgress(0);
        progressLabel.setText("0/" + total);
        onSkipAction(null);
    }

    @FXML
    private void onNextAction(ActionEvent event) {
        prevQuestion.setText(list.get(current).getQuestion());
        prevAnswer.setText(answer.getText());
        correctAnswer.setText(list.get(current).getAnswer());

        boolean right = list.get(current).getAnswer().equals(answer.getText());
        correctAnswer.setVisible(!right);
        correctLabel.setVisible(!right);

        if (!right) {
            errors++;
            prevAnswer.setStyle("-fx-text-fill: #C40000;");
        } else {
            prevAnswer.setStyle("-fx-text-fill: #00C400;");
        }

        // Remove the question from the pool if the question was correctly answered or there is no repetition
        if (right || !AppConfig.repeatWrong) {
            list.remove(current);
            progress.setProgress(++done/(double)total);
            progressLabel.setText(done + "/" + total);
            if (list.size() == 0) {
                onEndAction(event);
                return;
            }
        }
        onSkipAction(event);
    }

    @FXML
    private void onSkipAction(ActionEvent event) {
        answer.setText("");
        int previous = current;
        while (list.size() > 1 && current == previous)
            current = (int) (Math.random() * list.size());
        question.setText(list.get(current).getQuestion());
        answer.requestFocus();
    }

    @FXML
    private void onEndAction(ActionEvent event) {
        answer.setText("");
        answer.setDisable(true);
        question.setText("That's it!");
        feedback.getChildren().clear();
        feedback.setSpacing(5);
        feedback.setPadding(new Insets(5, 5, 5, 5));
        feedback.getChildren().add(new Label("Mistakes: " + errors));
        feedback.getChildren().add(new Label(String.format("Mark: %.2f%%", (total - errors) / (double) total * 100)));
    }
}
